@@ -1248,8 +1248,8 @@ forecast.legion <- function(object, h=10, #newdata=NULL, occurrence=NULL,
                          order.by=yForecastIndex);
     }
 
-    yForecast[] <- t(vForecasterWrap(matVt, matF, matW, nSeries, h,
-                                     Etype, Ttype, Stype, lagsModel));
+    yForecast[] <- t(vForecasterWrap(matVt, Matrix(matF, sparse=TRUE), Matrix(matW, sparse=TRUE),
+                                     nSeries, h, Etype, Ttype, Stype, lagsModel));
 
     if(cumulative){
         yForecast <- colSums(yForecast);
@@ -1261,7 +1261,8 @@ forecast.legion <- function(object, h=10, #newdata=NULL, occurrence=NULL,
     PI <- NA;
     if(interval!="none"){
         nElements <- length(lagsModel);
-        df <- obsInSample - nParam;
+        # Number of degrees of freedom per series
+        df <- obsInSample - nParam/nSeries;
 
         # In case of individual we use either Z distribution or Chebyshev inequality
         if(interval=="prediction"){
